@@ -1,7 +1,7 @@
 package Perinci::Sub::To::FishComplete;
 
-our $DATE = '2014-12-02'; # DATE
-our $VERSION = '0.02'; # VERSION
+our $DATE = '2015-09-04'; # DATE
+our $VERSION = '0.03'; # VERSION
 
 use 5.010001;
 use strict;
@@ -103,24 +103,78 @@ sub gen_fish_complete_from_meta {
         my $req_arg;
         for my $opt (split /, /, $opt0) {
             $opt =~ s/^--?//;
-            $opt =~ s/=.+// and $req_arg = 1;
+            $opt =~ s/=(.+)// and $req_arg = $1;
 
             my $cmd = $prefix;
             $cmd .= length($opt) > 1 ? " -l '$opt'" : " -s '$opt'";
             $cmd .= " -d ".shell_quote($ospec->{summary}) if $ospec->{summary};
 
-            if ($req_arg) {
-                $cmd .= " -r -f -a ".shell_quote("(begin; set -lx COMP_SHELL fish; set -lx COMP_LINE (commandline); set -lx COMP_POINT (commandline -C); ".shell_quote($cmdname)."; end)");
+          COMP_ARG_VAL:
+            {
+                last unless $req_arg;
+                $cmd .= " -r -f";
+                # check if completion is static, if yes then we can directly
+                # specify the entries to the shell
+                {
+                    require Perinci::Sub::Complete;
+
+                    my $compres;
+                    last if $ospec->{is_json} || $ospec->{is_yaml} ||
+                        $ospec->{is_base64};
+                    #say "D:Checking if $opt has static completion ...";
+                    if ($ospec->{arg}) {
+                        if ($req_arg =~ /\@/) {
+                            $compres =
+                                Perinci::Sub::Complete::complete_arg_elem(
+                                    arg=>$ospec->{arg}, ci=>1, index=>0,
+                                    meta=>$meta,
+                                );
+                        } else {
+                            $compres =
+                                Perinci::Sub::Complete::complete_arg_val(
+                                    arg=>$ospec->{arg}, ci=>1, meta=>$meta,
+                                );
+                        }
+                    } elsif ($ospec->{is_alias} &&
+                                 $ospec->{alias_spec}{schema}) {
+                        # cmdline alias which has schema
+                        require Data::Sah::Normalize;
+                        $compres = Perinci::Sub::Complete::complete_from_schema(
+                            ci=>1,
+                            schema=>Data::Sah::Normalize::normalize_schema($ospec->{alias_spec}{schema}),
+                        );
+                    } elsif ($ospec->{schema}) {
+                        # common opt which has schema
+                        require Data::Sah::Normalize;
+                        $compres = Perinci::Sub::Complete::complete_from_schema(
+                            ci=>1,
+                            schema=>Data::Sah::Normalize::normalize_schema($ospec->{schema}),
+                        );
+                    }
+
+                    last unless $compres;
+                    if ($compres->{static}) {
+                        # XXX description
+                        # XXX escape space
+                        my @words = map {ref($_) ? $_->{word}:$_}
+                            @{$compres->{words}};
+                        $cmd .= " -a ".shell_quote(join(" ", @words));
+                        last COMP_ARG_VAL;
+                    }
+                } # COMP_ARG_VAL
+                # completion is not static, delegate to the program when
+                # completing
+                $cmd .= " -a ".shell_quote("(begin; set -lx COMP_SHELL fish; set -lx COMP_LINE (commandline); set -lx COMP_POINT (commandline -C); ".shell_quote($cmdname)."; end)");
             }
             push @cmds, $cmd;
-        }
-    }
+        } # for opt
+    } # for opt0
 
     [200, "OK", join("", map {"$_\n"} @cmds)];
 }
 
 1;
-# ABSTRACT: Generate tab completion commands for the fish shell
+# ABSTRACT: From Rinci function metadata, generate tab completion commands for the fish shell
 
 __END__
 
@@ -130,11 +184,11 @@ __END__
 
 =head1 NAME
 
-Perinci::Sub::To::FishComplete - Generate tab completion commands for the fish shell
+Perinci::Sub::To::FishComplete - From Rinci function metadata, generate tab completion commands for the fish shell
 
 =head1 VERSION
 
-This document describes version 0.02 of Perinci::Sub::To::FishComplete (from Perl distribution Perinci-Sub-To-FishComplete), released on 2014-12-02.
+This document describes version 0.03 of Perinci::Sub::To::FishComplete (from Perl distribution Perinci-Sub-To-FishComplete), released on 2015-09-04.
 
 =head1 SYNOPSIS
 
@@ -143,6 +197,12 @@ This document describes version 0.02 of Perinci::Sub::To::FishComplete (from Per
  die "Failed: $res->[0] - $res->[1]" unless $res->[0] == 200;
  say $res->[2];
 
+=head1 SEE ALSO
+
+This module is used by L<Perinci::CmdLine>.
+
+L<Complete::Fish::Gen::FromGetoptLong>.
+
 =head1 FUNCTIONS
 
 
@@ -186,8 +246,6 @@ Pass per_arg_json=1 to Perinci::Sub::GetArgs::Argv.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -197,13 +255,7 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
-A script that can be fed to the fish shell (str)
-
-=head1 SEE ALSO
-
-This module is used by L<Perinci::CmdLine>.
-
-L<Complete::Fish::Gen::FromGetoptLong>.
+Return value: A script that can be fed to the fish shell (str)
 
 =head1 HOMEPAGE
 
@@ -227,7 +279,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by perlancar@cpan.org.
+This software is copyright (c) 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
